@@ -1,22 +1,23 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* htmlentity.c
+ *
+ * This file is part of the GtkHTML library.
  *
  * Copyright (C) 1999  Helix Code, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Library General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Library General Public License
+ * along with this library; see the file COPYING.LIB.  If not, write to
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  * Author: Ettore Perazzoli
  */
@@ -304,7 +305,7 @@ static EntityEntry entity_table[] = {
 static gint
 html_g_str_case_equal (gconstpointer v, gconstpointer v2)
 {
-	return strcasecmp ((const gchar*) v, (const gchar*)v2) == 0;
+	return g_ascii_strcasecmp ((const gchar*) v, (const gchar*)v2) == 0;
 }
 
 gulong
@@ -317,12 +318,9 @@ html_entity_parse (const gchar *s, guint len)
 		gint i;
 
 		ehash = g_hash_table_new (g_str_hash, html_g_str_case_equal);
-		g_hash_table_freeze (ehash);
 
 		for (i = 0; i < sizeof (entity_table) / sizeof (entity_table[0]); i++)
 			g_hash_table_insert (ehash, (gpointer) entity_table[i].str, GINT_TO_POINTER (entity_table[i].value));
-
-		g_hash_table_thaw (ehash);
 	}
 
 	if (len > 0) {
