@@ -17,8 +17,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include <config.h>
@@ -35,6 +35,7 @@
 #include "htmlcursor.h"
 #include "htmlengine.h"
 #include "htmlengine-edit.h"
+#include "htmlengine-edit-cut-and-paste.h"
 #include "htmlengine-save.h"
 #include "htmlframe.h"
 #include "htmlinterval.h"
@@ -46,6 +47,7 @@
 #include "htmlrule.h"
 #include "htmltype.h"
 #include "htmlsettings.h"
+#include "htmlutils.h"
 
 #include "gtkhtmldebug.h"
 
@@ -604,7 +606,7 @@ html_object_real_get_direction (HTMLObject *o)
 }
 
 static gboolean
-html_object_real_cursor_forward (HTMLObject *self, HTMLCursor *cursor)
+html_object_real_cursor_forward (HTMLObject *self, HTMLCursor *cursor, HTMLEngine *engine)
 {
 	gint len;
 
@@ -645,7 +647,7 @@ html_cursor_allow_zero_offset (HTMLCursor *cursor, HTMLObject *o)
 }
 
 static gboolean
-html_object_real_cursor_backward (HTMLObject *self, HTMLCursor *cursor)
+html_object_real_cursor_backward (HTMLObject *self, HTMLCursor *cursor, HTMLEngine *engine)
 {
 	g_assert (self);
 	g_assert (cursor->object == self);
@@ -726,6 +728,15 @@ html_object_real_cursor_left (HTMLObject *self, HTMLPainter *painter, HTMLCursor
 	return FALSE;
 }
 
+static gboolean
+html_object_real_backspace (HTMLObject *self, HTMLCursor *cursor, HTMLEngine *engine)
+{
+	html_cursor_backward (cursor, engine);
+	html_engine_delete (engine);
+
+	return TRUE;
+}
+
 static int
 html_object_real_get_right_edge_offset (HTMLObject *o, HTMLPainter *painter, int offset)
 {
@@ -813,6 +824,7 @@ html_object_class_init (HTMLObjectClass *klass,
 	klass->cursor_backward = html_object_real_cursor_backward;
 	klass->cursor_left = html_object_real_cursor_left;
 	klass->cursor_right = html_object_real_cursor_right;
+	klass->backspace = html_object_real_backspace;
 	klass->get_right_edge_offset = html_object_real_get_right_edge_offset;
 	klass->get_left_edge_offset = html_object_real_get_left_edge_offset;
 }
@@ -1160,11 +1172,16 @@ gchar *
 html_object_get_complete_url (HTMLObject *o, gint offset)
 {
 	const gchar *url, *target;
+	gchar *url2, *url3;
 
 	url = html_object_get_url (o, offset);
 	target = html_object_get_target (o, offset);
-	return url || target ? g_strconcat (url ? url : "#", url ? (target && *target ? "#" : NULL) : target,
-					      url ? target : NULL, NULL) : NULL;
+	url2 = url || target ? g_strconcat (url ? url : "#", url ? (target && *target ? "#" : NULL) : target,
+					    url ? target : NULL, NULL) : NULL;
+	url3 = html_utils_maybe_unescape_amp (url2);
+	g_free (url2);
+
+	return url3;
 }
 
 const gchar *
@@ -1239,6 +1256,10 @@ html_object_get_cursor (HTMLObject *self,
 			gint *x2, gint *y2)
 {
 	(* HO_CLASS (self)->get_cursor) (self, painter, offset, x1, y1, x2, y2);
+
+	if (*y2 - *y1 > *y2 - self->ascent)
+		*y2 = *y1 + 20;
+
 	if (!html_object_is_text (self) && *y2 - *y1 < 10) {
 		gint missing = 10 - (*y2 - *y1);
 
@@ -1546,15 +1567,15 @@ html_object_tail_not_slave (HTMLObject *self)
 }
 
 gboolean
-html_object_cursor_forward (HTMLObject *self, HTMLCursor *cursor)
+html_object_cursor_forward (HTMLObject *self, HTMLCursor *cursor, HTMLEngine *engine)
 {
-	return (* HO_CLASS (self)->cursor_forward) (self, cursor);
+	return (* HO_CLASS (self)->cursor_forward) (self, cursor, engine);
 }
 
 gboolean
-html_object_cursor_backward (HTMLObject *self, HTMLCursor *cursor)
+html_object_cursor_backward (HTMLObject *self, HTMLCursor *cursor, HTMLEngine *engine)
 {
-	return (* HO_CLASS (self)->cursor_backward) (self, cursor);
+	return (* HO_CLASS (self)->cursor_backward) (self, cursor, engine);
 }
 
 gboolean
@@ -1569,6 +1590,12 @@ html_object_cursor_left (HTMLObject *self, HTMLPainter *painter, HTMLCursor *cur
 	return (* HO_CLASS (self)->cursor_left) (self, painter, cursor);
 }
 
+gboolean
+html_object_backspace (HTMLObject *self, HTMLCursor *cursor, HTMLEngine *engine)
+{
+	return (* HO_CLASS (self)->backspace) (self, cursor, engine);
+}
+
 /*********************
  * movement on leafs
  */
